"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _global = _interopRequireDefault(require("global"));

var _ansiColors = require("ansi-colors");

var _register = _interopRequireDefault(require("babel-plugin-require-context-hook/register"));

var _globalJsdom = _interopRequireDefault(require("global-jsdom"));

var _reactElementToJsxString = _interopRequireDefault(require("react-element-to-jsx-string"));

var _coreCommon = require("@storybook/core-common");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable global-require, no-console, import/no-dynamic-require, @typescript-eslint/no-var-requires */
// Based on https://github.com/storybookjs/storybook/blob/next/addons/storyshots/storyshots-core/src/frameworks/configure.ts
(0, _register.default)();
(0, _globalJsdom.default)();

const storybook = require('@storybook/react');

const isFile = file => {
  try {
    return _fs.default.lstatSync(file).isFile();
  } catch (err) {
    return false;
  }
};

const extensions = ['ts', 'tsx', 'js', 'jsx'];

const resolveFile = (configDir, filenames) => filenames.flatMap(filename => extensions.map(ext => _path.default.join(configDir, `${filename}.${ext}`))).find(isFile) || false;

const getPreviewFile = configDir => resolveFile(configDir, ['preview', 'config']);

const getMainFile = configDir => resolveFile(configDir, ['main']);

const getConfigPathParts = input => {
  const configDir = _path.default.resolve(input);

  if (_fs.default.lstatSync(configDir).isDirectory()) {
    const output = {};
    const preview = getPreviewFile(configDir);
    const main = getMainFile(configDir);

    if (preview) {
      output.preview = preview;
    }

    if (main) {
      const {
        stories = []
      } = require(main);

      const workingDir = process.cwd();
      output.stories = stories.map(entry => {
        const specifier = (0, _coreCommon.normalizeStoriesEntry)(entry, {
          configDir,
          workingDir
        });
        const {
          path: basePath,
          recursive,
          match
        } = (0, _coreCommon.toRequireContext)(specifier); // eslint-disable-next-line no-underscore-dangle

        return _global.default.__requireContext(workingDir, basePath, recursive, match);
      });
    }

    return output;
  }

  return {
    preview: configDir
  };
};

const configure = options => {
  const {
    configDir
  } = options;
  const {
    preview,
    stories
  } = getConfigPathParts(configDir);

  if (preview) {
    const {
      parameters,
      decorators,
      globals,
      globalTypes,
      argsEnhancers,
      argTypesEnhancers
    } = require(preview);

    if (decorators) {
      decorators.forEach(decorator => storybook.addDecorator(decorator));
    }

    if (parameters || globals || globalTypes) {
      storybook.addParameters({ ...parameters,
        globals,
        globalTypes
      });
    }

    if (argsEnhancers) {
      argsEnhancers.forEach(enhancer => storybook.addArgsEnhancer(enhancer));
    }

    if (argTypesEnhancers) {
      argTypesEnhancers.forEach(enhancer => storybook.addArgTypesEnhancer(enhancer));
    }
  }

  if (stories?.length > 0) {
    storybook.configure(stories, false, false);
  }
};

var _default = (configDir, {
  outFile
}) => {
  configure({
    configDir
  });
  const snippets = storybook.raw().map(({
    kind: group,
    name,
    storyFn,
    parameters: {
      playroom
    } = {}
  }) => {
    const {
      disable,
      reactElementToJSXStringOptions
    } = (0, _utils.getOptions)(playroom);

    if (disable) {
      return undefined;
    }

    console.log(`Generating ${(0, _ansiColors.yellow)([group, name].join('/'))} snippet...`);
    const code = (0, _reactElementToJsxString.default)(storyFn(), reactElementToJSXStringOptions);
    return {
      group,
      name,
      code
    };
  }).filter(Boolean);
  const count = snippets.length;

  _fs.default.writeFileSync(_path.default.resolve(process.cwd(), outFile), JSON.stringify(snippets, null, 2));

  console.log(`\n${count} ${count === 1 ? 'snippet' : 'snippets'} generated.`);
};

exports.default = _default;